#!/usr/bin/env python

# standard lib
import warnings

# local
from ..utils import PyDyUserWarning
from .shapes import Shape
from .visualization_frame import VisualizationFrame

__all__ = ['PerspectiveCamera', 'OrthoGraphicCamera']

warnings.simplefilter('once', PyDyUserWarning)


class PerspectiveCamera(VisualizationFrame):
    """Creates a perspective camera for use in a scene. The camera is inherited
    from ``VisualizationFrame``, and thus behaves similarly. It can be attached
    to dynamics objects, hence we can get a moving camera. All the
    transformation matrix generation methods are applicable to a
    ``PerspectiveCamera``.

    """

    def __init__(self, *args, **kwargs):
        """Initialises a PerspectiveCamera object. To initialize a
        PerspectiveCamera, one needs to supply a name (optional), a reference
        frame, a point, field of view (fov) (optional), near plane distance
        (optional) and far plane distance (optional).

        Like ``VisualizationFrame``, it can also be initialized using one of
        these three argument sequences:

        Rigidbody
        ``PerspectiveCamera(rigid_body)``
        ReferenceFrame, Point
        ``PerspectiveCamera(ref_frame, point)``
        ReferenceFrame, Particle
        ``PerspectiveCamera(ref_frame, particle)``

        Note that you can also supply and optional name as the first positional
        argument, e.g.::

           ``PerspectiveCamera('camera_name', rigid_body)``

        Additional optional keyword arguments are below:

        Parameters
        ==========
        fov : float, default=45.0
            Field Of View, It determines the angle between the top and bottom
            of the viewable area (in degrees).
        near : float
            The distance of near plane of the PerspectiveCamera. All objects
            closer to this distance are not displayed.
        far : int or float
            The distance of far plane of the PerspectiveCamera. All objects
            farther than this distance are not displayed.

        Examples
        ========

        >>> from sympy import symbols
        >>> from sympy.physics.mechanics import (ReferenceFrame, Point,
        ...                                      RigidBody, Particle,
        ...                                      inertia)
        >>> from pydy.viz import PerspectiveCamera
        >>> I = ReferenceFrame('I')
        >>> O = Point('O')

        >>> # initializing with reference frame, point
        >>> camera1 = PerspectiveCamera('frame1', I, O)

        >>> # Initializing with a RigidBody
        >>> Ixx, Iyy, Izz, mass = symbols('Ixx Iyy Izz mass')
        >>> i = inertia(I, Ixx, Iyy, Izz)
        >>> rbody = RigidBody('rbody', O, I, mass, (inertia, O))
        >>> camera2 = PerspectiveCamera('frame2', rbody)

        >>> # initializing with Particle, reference_frame
        >>> Pa = Particle('Pa', O, mass)
        >>> camera3 = PerspectiveCamera('frame3', I, Pa)

        """

        # NOTE: This allows us to use inhertiance even though cameras don't
        # need a shape. In the future, this could be a camera shape that could
        # be made visible in the scene (only important for multiple cameras).
        args = list(args) + [Shape()]

        super(PerspectiveCamera, self).__init__(*args)

        self.fov = 45.0
        self.near = 1.0
        self.far = 1000.0

        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        return 'PerspectiveCamera: ' + self.name

    def __repr__(self):
        return 'PerspectiveCamera'

    @property
    def fov(self):
        return self._fov

    @fov.setter
    def fov(self, new_fov):
        self._fov = float(new_fov)

    @property
    def near(self):
        return self._near

    @near.setter
    def near(self, new_near):
        self._near = float(new_near)

    @property
    def far(self):
        return self._far

    @far.setter
    def far(self, new_far):
        self._far = float(new_far)

    def generate_scene_dict(self, **kwargs):
        """This method generates information for a static visualization in the
        initial conditions, in the form of dictionary. This contains camera
        parameters followed by an init_orientation key.

        Before calling this method, all the transformation matrix generation
        methods should be called, or it will give an error.

        Returns
        =======
        A dict with following Keys:

        1. name: name for the camera
        2. fov: Field of View value of the camera
        3. near: near value of the camera
        4. far: far value of the camera
        5. init_orientation: Initial orientation of the camera

        """
        scene_dict = super(PerspectiveCamera, self).generate_scene_dict(**kwargs)
        scene_dict[id(self)]['type'] = self.__class__.__name__
        scene_dict[id(self)]['fov'] = self.fov
        scene_dict[id(self)]['near'] = self.near
        scene_dict[id(self)]['far'] = self.far

        return scene_dict


class OrthoGraphicCamera(VisualizationFrame):
    """Creates a orthographic camera for use in a scene. The camera is
    inherited from ``VisualizationFrame``, and thus behaves similarly. It can
    be attached to dynamics objects, hence we can get a moving camera. All the
    transformation matrix generation methods are applicable to a
    ``OrthoGraphicCameraCamera``.


    """

    def __init__(self, *args, **kwargs):
        """Initialises a OrthoGraphicCameraCamera object. To initialize a
        OrthoGraphicCameraCamera, one needs to supply a name (optional), a
        reference frame, a point, field of view (fov) (optional), near plane
        distance (optional) and far plane distance (optional).

        Like ``VisualizationFrame``, it can also be initialized using one of
        these three argument sequences:

        Rigidbody
           ``OrthoGraphicCameraCamera(rigid_body)``
        ReferenceFrame, Point
           ``OrthoGraphicCameraCamera(ref_frame, point)``
        ReferenceFrame, Particle
           ``OrthoGraphicCameraCamera(ref_frame, particle)``

        Note that you can also supply and optional name as the first positional
        argument, e.g.::

            OrthoGraphicCameraCamera('camera_name', rigid_body)

        Additional optional keyword arguments are below:

        Parameters
        ==========
        near : float
            The distance of near plane of the OrthoGraphicCameraCamera. All
            objects closer to this distance are not displayed.
        far : int or float
            The distance of far plane of the OrthoGraphicCameraCamera. All
            objects farther than this distance are not displayed.

        Examples
        ========

        >>> from sympy import symbols
        >>> from sympy.physics.mechanics import (ReferenceFrame, Point,
        ...                                      RigidBody, Particle,
        ...                                      inertia)
        >>> from pydy.viz import OrthoGraphicCameraCamera
        >>> I = ReferenceFrame('I')
        >>> O = Point('O')

        >>> # initializing with reference frame, point
        >>> camera1 = OrthoGraphicCameraCamera('frame1', I, O)

        >>> # Initializing with a RigidBody
        >>> Ixx, Iyy, Izz, mass = symbols('Ixx Iyy Izz mass')
        >>> i = inertia(I, Ixx, Iyy, Izz)
        >>> rbody = RigidBody('rbody', O, I, mass, (inertia, O))
        >>> camera2 = OrthoGraphicCameraCamera('frame2', rbody)

        >>> # initializing with Particle, reference_frame
        >>> Pa = Particle('Pa', O, mass)
        >>> camera3 = OrthoGraphicCameraCamera('frame3', I, Pa)

        """
        # NOTE: This allows us to use inhertiance even though cameras don't
        # need a shape. In the future, this could be a camera shape that could
        # be made visible in the scene (only important for multiple cameras).
        args = list(args) + [Shape()]

        super(OrthoGraphicCamera, self).__init__(*args)

        self.near = 1.0
        self.far = 1000.0

        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        return self.__class__.__name__ + ': ' + self.name

    def __repr__(self):
        return self.__class__.__name__

    @property
    def near(self):
        return self._near

    @near.setter
    def near(self, new_near):
        self._near = float(new_near)

    @property
    def far(self):
        return self._far

    @far.setter
    def far(self, new_far):
        self._far = float(new_far)

    def generate_scene_dict(self, **kwargs):
        """
        This method generates information for a static visualization in the
        initial conditions, in the form of dictionary. This contains camera
        parameters followed by an init_orientation Key.

        Returns
        =======
        scene_dict : dictionary
           A dict with following Keys:

              1. name: name for the camera
              2. near: near value of the camera
              3. far: far value of the camera
              4. init_orientation: Initial orientation of the camera

        """
        scene_dict = super(OrthoGraphicCamera, self).generate_scene_dict(**kwargs)
        scene_dict[id(self)]['type'] = self.__class__.__name__
        scene_dict[id(self)]['near'] = self.near
        scene_dict[id(self)]['far'] = self.far

        return scene_dict



#!/usr/bin/env python

# external
from sympy import sin, cos, symbols, pi
from sympy.physics.mechanics import (dynamicsymbols, ReferenceFrame, Point,
                                     RigidBody, Particle, inertia)
from numpy import radians
from numpy.testing import assert_allclose

# local
from ..shapes import Cylinder
from ..visualization_frame import VisualizationFrame
from ..camera import PerspectiveCamera, OrthoGraphicCamera
from ..scene import Scene
from ..light import PointLight


class TestVisualizationFrameScene(object):

    def __init__(self):

        self.coords = dynamicsymbols('q:3')
        self.speeds = dynamicsymbols('p:3')

        self.dynamic = list(self.coords) + list(self.speeds)
        self.states = ([radians(45) for x in self.coords] +
                       [radians(30) for x in self.speeds])

        self.inertial_ref_frame = ReferenceFrame('I')
        self.A = self.inertial_ref_frame.orientnew('A', 'space',
                                                   self.coords, 'XYZ')
        self.B = self.A.orientnew('B', 'space', self.speeds, 'XYZ')
        self.camera_ref_frame = \
            self.inertial_ref_frame.orientnew('C', 'space', (pi / 2, 0, 0),
                                              'XYZ')

        self.origin = Point('O')
        self.P1 = self.origin.locatenew('P1',
                                        10 * self.inertial_ref_frame.x +
                                        10 * self.inertial_ref_frame.y +
                                        10 * self.inertial_ref_frame.z)
        self.P2 = self.P1.locatenew('P2',
                                    10 * self.inertial_ref_frame.x +
                                    10 * self.inertial_ref_frame.y +
                                    10 * self.inertial_ref_frame.z)

        self.point_list1 = [[2, 3, 1], [4, 6, 2], [5, 3, 1], [5, 3, 6]]
        self.point_list2 = [[3, 1, 4], [3, 8, 2], [2, 1, 6], [2, 1, 1]]

        self.shape1 = Cylinder(1.0, 1.0)
        self.shape2 = Cylinder(1.0, 1.0, name='booger')

        self.Ixx, self.Iyy, self.Izz = symbols('Ixx, Iyy, Izz')
        self.mass = symbols('m')
        self.parameters = [self.Ixx, self.Iyy, self.Izz, self.mass]
        self.param_vals = [2.0, 3.0, 4.0, 5.0]
        self.inertia = inertia(self.A, self.Ixx, self.Iyy, self.Izz)

        self.rigid_body = RigidBody('rigid_body1', self.P1, self.A,
                                    self.mass, (self.inertia, self.P1))

        self.global_frame1 = VisualizationFrame('global_frame1', self.A,
                                                self.P1, self.shape1)

        self.global_frame2 = VisualizationFrame('global_frame2', self.B,
                                                self.P2, self.shape2)

        self.scene1 = Scene(self.inertial_ref_frame, self.origin,
                            (self.global_frame1, self.global_frame2),
                            name='scene')

        self.particle = Particle('particle1', self.P1, self.mass)

        q = self.coords
        c = cos
        s = sin
        # Here is the dragon ..
        self.transformation_matrix = [
            [c(q[1])*c(q[2]), s(q[2])*c(q[1]), -s(q[1]), 0],
            [s(q[0])*s(q[1])*c(q[2]) - s(q[2])*c(q[0]),
             s(q[0])*s(q[1])*s(q[2]) + c(q[0])*c(q[2]),
             s(q[0])*c(q[1]), 0],
            [s(q[0])*s(q[2]) + s(q[1])*c(q[0])*c(q[2]),
             -s(q[0])*c(q[2]) + s(q[1])*s(q[2])*c(q[0]),
             c(q[0])*c(q[1]), 0],
            [10, 10, 10, 1]]

    def test_vframe_with_rframe(self):
        self.frame1 = VisualizationFrame('frame1', self.inertial_ref_frame,
                                         self.origin, self.shape1)

        assert self.frame1.name == 'frame1'
        assert self.frame1.reference_frame == self.inertial_ref_frame
        assert self.frame1.origin == self.origin
        assert self.frame1.shape is self.shape1

        self.frame1.name = 'frame1_'
        assert self.frame1.name == 'frame1_'

        self.frame1.reference_frame = self.A
        assert self.frame1.reference_frame == self.A

        self.frame1.origin = self.P1
        assert self.frame1.origin == self.P1

        self.frame1.shape = self.shape2
        assert self.frame1.shape is self.shape2

        tr_mat = self.frame1.generate_transformation_matrix(
            self.inertial_ref_frame, self.origin).tolist()
        assert tr_mat == self.transformation_matrix

    def test_vframe_with_rbody(self):
        self.frame2 = VisualizationFrame('frame2', self.rigid_body,
                                         self.shape1)

        assert self.frame2.name == 'frame2'
        assert self.frame2.reference_frame == self.A
        assert self.frame2.origin == self.P1
        assert self.frame2.shape == self.shape1

        self.frame2.name = 'frame2_'
        assert self.frame2.name == 'frame2_'

        self.frame2.reference_frame = self.B
        assert self.frame2.reference_frame == self.B

        self.frame2.origin = self.P2
        assert self.frame2.origin == self.P2

        self.frame2.shape = self.shape2
        assert self.frame2.shape is self.shape2

        self.frame2.reference_frame = self.A
        self.frame2.origin = self.P1
        tr_mat = self.frame2.generate_transformation_matrix(
            self.inertial_ref_frame, self.origin).tolist()
        assert tr_mat == self.transformation_matrix

    def test_vframe_with_particle(self):

        self.frame3 = VisualizationFrame('frame3', self.A, self.particle,
                                         self.shape1)

        assert self.frame3.name == 'frame3'
        assert self.frame3.reference_frame == self.A
        assert self.frame3.origin == self.P1
        assert self.frame3.shape is self.shape1

        self.frame3.name = 'frame3_'
        assert self.frame3.name == 'frame3_'

        self.frame3.reference_frame = self.B
        assert self.frame3.reference_frame == self.B

        self.frame3.origin = self.P2
        assert self.frame3.origin == self.P2

        self.frame3.shape = self.shape2
        assert self.frame3.shape is self.shape2

        self.frame3.reference_frame = self.A
        self.frame3.origin = self.P1
        tr_mat = self.frame3.generate_transformation_matrix(
            self.inertial_ref_frame, self.origin).tolist()
        assert tr_mat == self.transformation_matrix

    def test_vframe_without_name(self):

        self.frame4 = VisualizationFrame(self.inertial_ref_frame, self.origin,
                                         self.shape1)

        assert self.frame4.name == 'unnamed'
        # To check if referenceframe and origin are defined properly without
        # name arg
        assert self.frame4.reference_frame == self.inertial_ref_frame
        assert self.frame4.origin == self.origin
        assert self.frame4.shape is self.shape1

        self.frame4.name = 'frame1_'
        assert self.frame4.name == 'frame1_'

    def test_numeric_transform(self):

        self.list1 = [[0.5000000000000001,
                       0.5,
                       -0.7071067811865475,
                       0.0,
                       -0.14644660940672627,
                       0.8535533905932737,
                       0.5,
                       0.0,
                       0.8535533905932737,
                       -0.14644660940672627,
                       0.5000000000000001,
                       0.0,
                       10.0,
                       10.0,
                       10.0,
                       1.0]]

        self.list2 = [[-0.11518993731879767,
                       0.8178227645734215,
                       -0.563823734943801,
                       0.0,
                       0.1332055011661179,
                       0.5751927992738988,
                       0.8070994598700584,
                       0.0,
                       0.984371663956036,
                       0.017865313009926137,
                       -0.17519491371464685,
                       0.0,
                       20.0,
                       20.0,
                       20.0,
                       1.0]]

        times = [1.0, 2.0, 3.0, 4.0]

        self.global_frame1.generate_transformation_matrix(
            self.inertial_ref_frame, self.origin)
        self.global_frame1.generate_numeric_transform_function(
            self.dynamic, self.parameters)

        assert_allclose(self.global_frame1.evaluate_transformation_matrix(
            self.states, self.param_vals), self.list1)

        self.global_frame2.generate_transformation_matrix(
            self.inertial_ref_frame, self.origin)
        self.global_frame2.generate_numeric_transform_function(
            self.dynamic, self.parameters)

        assert_allclose(self.global_frame2.evaluate_transformation_matrix(
            self.states, self.param_vals), self.list2)

        try:
            import pythreejs
        except ImportError:
            print('pythreejs not installed, skipping related tests.')
        else:
            del pythreejs
            self.global_frame2._create_keyframetrack(
                times, self.states, self.param_vals,
                constant_map=dict(zip(self.parameters, self.param_vals)))
            self.global_frame2._track.name == 'scene/booger.matrix'
            assert_allclose(self.global_frame2._track.times, times)

    def test_perspective_camera(self):

        # Camera is a subclass of VisualizationFrame, but without any specific
        # shape attached. We supply only ReferenceFrame,Point to camera. and it
        # inherits methods from VisualizationFrame

        camera = PerspectiveCamera('camera', self.rigid_body, fov=45, near=1,
                                   far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.A
        assert camera.origin == self.P1
        assert camera.fov == 45
        assert camera.near == 1
        assert camera.far == 1000

        camera = PerspectiveCamera('camera', self.inertial_ref_frame,
                                   self.particle, fov=45, near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.inertial_ref_frame
        assert camera.origin == self.P1
        assert camera.fov == 45
        assert camera.near == 1
        assert camera.far == 1000

        camera = PerspectiveCamera('camera', self.inertial_ref_frame,
                                   self.origin, fov=45, near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.inertial_ref_frame
        assert camera.origin == self.origin
        assert camera.fov == 45
        assert camera.near == 1
        assert camera.far == 1000

        camera.name = 'camera1'
        assert camera.name == 'camera1'
        assert camera.__str__() == 'PerspectiveCamera: camera1'
        assert camera.__repr__() == 'PerspectiveCamera'

        camera.reference_frame = self.A
        assert camera.reference_frame == self.A

        camera.origin = self.P1
        assert camera.origin == self.P1

        camera.fov = 30
        assert camera.fov == 30

        camera.near = 10
        assert camera.near == 10

        camera.far = 500
        assert camera.far == 500

        camera = PerspectiveCamera('camera', self.camera_ref_frame,
                                   self.origin, fov=45, near=1, far=1000)
        camera.generate_transformation_matrix(self.inertial_ref_frame,
                                              self.origin)
        camera.generate_numeric_transform_function(self.dynamic,
                                                   self.parameters)
        viz_matrix = camera.evaluate_transformation_matrix(self.states,
                                                           self.param_vals)
        expected_viz_matrix = [[1., 0., 0., 0.,
                                0., 0., 1., 0.,
                                0., -1., 0., 0.,
                                0., 0., 0., 1.]]
        assert_allclose(viz_matrix, expected_viz_matrix)

        scene_dict = camera.generate_scene_dict()

        assert scene_dict[id(camera)]['type'] == 'PerspectiveCamera'
        assert scene_dict[id(camera)]['fov'] == 45
        assert scene_dict[id(camera)]['near'] == 1
        assert scene_dict[id(camera)]['far'] == 1000
        assert scene_dict[id(camera)]['name'] == 'camera'
        assert scene_dict[id(camera)]['init_orientation'] == \
            [1.,  0.,  0.,  0.,
             0.,  0.,  1.,  0.,
             0., -1.,  0.,  0.,
             0.,  0.,  0.,  1.]

        camera1 = PerspectiveCamera(self.inertial_ref_frame, self.origin)
        assert camera1.name == 'unnamed'
        assert camera1.reference_frame == self.inertial_ref_frame
        assert camera1.origin == self.origin
        assert camera1.fov == 45
        assert camera1.near == 1
        assert camera1.far == 1000

    def test_orthographic_camera(self):

        # As compared to Perspective Camera, Orthographic Camera doesn't have
        # fov, instead the left,right,top and bottom faces are adjusted by the
        # Scene width, and height

        # Testing with rigid_body
        camera = OrthoGraphicCamera('camera', self.rigid_body, near=1,
                                    far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.A
        assert camera.origin == self.P1
        assert camera.near == 1
        assert camera.far == 1000

        # Testing with reference_frame, particle
        camera = OrthoGraphicCamera('camera', self.inertial_ref_frame,
                                    self.particle, near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.inertial_ref_frame
        assert camera.origin == self.P1
        assert camera.near == 1
        assert camera.far == 1000

        # Testing with reference_frame, point ...
        camera = OrthoGraphicCamera('camera', self.inertial_ref_frame,
                                    self.origin, near=1, far=1000)

        assert camera.name == 'camera'
        assert camera.reference_frame == self.inertial_ref_frame
        assert camera.origin == self.origin
        assert camera.near == 1
        assert camera.far == 1000

        camera.name = 'camera1'
        assert camera.name == 'camera1'
        assert camera.__str__() == 'OrthoGraphicCamera: camera1'
        assert camera.__repr__() == 'OrthoGraphicCamera'

        camera.reference_frame = self.A
        assert camera.reference_frame == self.A

        camera.origin = self.P1
        assert camera.origin == self.P1

        camera.near = 10
        assert camera.near == 10

        camera.far = 500
        assert camera.far == 500

        camera1 = OrthoGraphicCamera(self.inertial_ref_frame, self.origin)
        assert camera1.name == 'unnamed'
        assert camera1.reference_frame == self.inertial_ref_frame
        assert camera1.origin == self.origin
        assert camera1.near == 1
        assert camera1.far == 1000

        camera = OrthoGraphicCamera('camera', self.camera_ref_frame,
                                    self.origin, near=1, far=1000)
        camera.generate_transformation_matrix(self.inertial_ref_frame,
                                              self.origin)
        camera.generate_numeric_transform_function(self.dynamic,
                                                   self.parameters)
        viz_matrix = camera.evaluate_transformation_matrix(self.states,
                                                           self.param_vals)
        expected_viz_matrix = [[1., 0., 0., 0.,
                                0., 0., 1., 0.,
                                0., -1., 0., 0.,
                                0., 0., 0., 1.]]
        assert_allclose(viz_matrix, expected_viz_matrix)

        scene_dict = camera.generate_scene_dict()

        assert scene_dict[id(camera)]['type'] == 'OrthoGraphicCamera'
        assert scene_dict[id(camera)]['near'] == 1
        assert scene_dict[id(camera)]['far'] == 1000
        assert scene_dict[id(camera)]['name'] == 'camera'
        assert scene_dict[id(camera)]['init_orientation'] == \
            [1.,  0.,  0.,  0.,
             0.,  0.,  1.,  0.,
             0., -1.,  0.,  0.,
             0.,  0.,  0.,  1.]

    def test_point_light(self):

        # Testing with rigid-body ..
        light = PointLight('light', self.rigid_body, color='blue')

        assert light.name == 'light'
        assert light.reference_frame == self.A
        assert light.origin == self.P1
        assert light.color == 'blue'

        # Testing with reference_frame, particle ..
        light = PointLight('light', self.inertial_ref_frame, self.particle,
                           color='blue')

        assert light.name == 'light'
        assert light.reference_frame == self.inertial_ref_frame
        assert light.origin == self.P1
        assert light.color == 'blue'

        # Testing with reference_frame, point ..
        light = PointLight('light', self.inertial_ref_frame, self.origin,
                           color='blue')

        assert light.name == 'light'
        assert light.reference_frame == self.inertial_ref_frame
        assert light.origin == self.origin
        assert light.color == 'blue'

        light.name = 'light1'
        assert light.name == 'light1'
        assert light.__str__() == 'PointLight: light1'
        assert light.__repr__() == 'PointLight'

        light.reference_frame = self.A
        assert light.reference_frame == self.A

        light.origin = self.P1
        assert light.origin == self.P1

        light.color = 'red'
        assert light.color == 'red'

        # Test unnamed
        light1 = PointLight(self.inertial_ref_frame, self.origin)
        assert light1.name == 'unnamed'
        assert light1.reference_frame == self.inertial_ref_frame
        assert light1.origin == self.origin
        assert light1.color == 'white'

    def test_scene_init(self):

        self.scene2 = Scene(self.inertial_ref_frame, self.origin,
                            self.global_frame1, self.global_frame2,
                            name='scene')

        assert self.scene2.name == 'scene'
        assert self.scene2.reference_frame == self.inertial_ref_frame
        assert self.scene2.origin == self.origin
        assert self.scene2.visualization_frames[0] is self.global_frame1
        assert self.scene2.visualization_frames[1] is self.global_frame2

        self.scene2.name = 'scene1'
        assert self.scene2.name == 'scene1'

        self.scene2.reference_frame = self.A
        assert self.scene2.reference_frame == self.A
